import six
import yaml
from django.db import models
from django.core.exceptions import ValidationError
from .serializers import OrderedDumper, OrderedLoader


class YAMLField(models.TextField):

    def from_db_value(self, value, expression, connection, context):
        return self.to_python(value)

    def to_python(self, value):
        """
        Convert our YAML string to a Python object
        after we load it from the DB.
        """
        if value == "":
            return None
        try:
            if isinstance(value, six.string_types):
                return yaml.load(value, OrderedLoader)
        except ValueError:
            raise ValidationError("Enter valid YAML")
        return value

    def get_prep_value(self, value):
        """
        Convert our Python object to a string of YAML before we save.
        """
        if not value or value == "":
            return ""
        if isinstance(value, (dict, list)):
            value = yaml.dump(
                value,
                Dumper=OrderedDumper,
                default_flow_style=False
            )
        return value

    def value_from_object(self, obj):
        """
        Returns the value of this field in the given model instance.

        We need to override this so that the YAML comes out properly formatted
        in the admin widget.
        """
        value = getattr(obj, self.attname)
        if not value or value == "":
            return value
        return yaml.dump(
            value,
            Dumper=OrderedDumper,
            default_flow_style=False
        )



from __future__ import absolute_import
import json
import collections

from django.core.exceptions import ValidationError
from django.db import models
from .fields import YAMLField
from django.test import TestCase
from django.db import connection


class YAMLModel(models.Model):
    yaml = YAMLField()


class YAMLFieldTest(TestCase):
    """
    YAMLField Wrapper Tests
    """
    def test_yaml_field_create(self):
        """
        Test saving a YAML object in our YAMLField
        """
        yaml_obj = {
            "item_1": "this is a yaml blah",
            "blergh": "hey, hey, hey"
        }
        obj = YAMLModel.objects.create(yaml=yaml_obj)
        new_obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(new_obj.yaml, yaml_obj)

    def test_yaml_field_modify(self):
        """
        Test modifying a YAML object in our YAMLField
        """
        yaml_obj_1 = {'a': 1, 'b': 2}
        yaml_obj_2 = {'a': 3, 'b': 4}
        obj = YAMLModel.objects.create(yaml=yaml_obj_1)
        self.failUnlessEqual(obj.yaml, yaml_obj_1)
        obj.yaml = yaml_obj_2
        self.failUnlessEqual(obj.yaml, yaml_obj_2)
        obj.save()
        self.failUnlessEqual(obj.yaml, yaml_obj_2)
        self.assert_(obj)

    def test_yaml_field_load(self):
        """
        Test loading a YAML object from the DB
        """
        yaml_obj_1 = {'a': 1, 'b': 2}
        obj = YAMLModel.objects.create(yaml=yaml_obj_1)
        new_obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(new_obj.yaml, yaml_obj_1)

    def test_yaml_list(self):
        """
        Test storing a yaml list
        """
        yaml_obj = ["my", "list", "of", 1, "objs", {"hello": "there"}]
        obj = YAMLModel.objects.create(yaml=yaml_obj)
        new_obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(new_obj.yaml, yaml_obj)

    def test_blank_yaml_field(self):
        # When there's no yaml...
        obj = YAMLModel.objects.create()
        # It should come out to python as None
        new_obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(new_obj.yaml, None)
        # But be stored as an empty string in the database
        cursor = connection.cursor()
        cursor.execute("SELECT * FROM yamlfield_yamlmodel;")
        row = cursor.fetchone()
        self.failUnlessEqual(row[1], "")

    def test_bad_yaml_to_python(self):
        with self.assertRaises(ValidationError) as exc:
            YAMLField().to_python('2013-01-65')
        self.assertEqual("Enter valid YAML", exc.exception.message)

    def test_value_from_object(self):
        yaml_obj = {
            "item_1": "this is a yaml blah",
            "blergh": "hey, hey, hey"
        }
        obj = YAMLModel.objects.create(yaml=yaml_obj)
        YAMLModel._meta.get_field('yaml').value_from_object(obj)

        obj2 = YAMLModel.objects.create(yaml='')
        YAMLModel._meta.get_field('yaml').value_from_object(obj2)

    def test_ordereddict(self):
        """
        Test storing an OrderedDict
        """
        ordered_data = collections.OrderedDict(
            [('one', 1), ('two', 2), ('five', 5), ('seven', 7), ('ten', 10)]
        )
        obj = YAMLModel.objects.create(yaml=[ordered_data])
        new_obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(new_obj.yaml, [ordered_data])
        ordered_data.pop('seven')
        ordered_data['seven'] = 7
        self.assertEqual(dict(ordered_data), dict(new_obj.yaml[0]))
        self.failIfEqual(new_obj.yaml, [ordered_data])

    def test_nested_ordereddict(self):
        """
        Test an ordered dict nested in the YAML data
        """
        data = """{
          "pk": 11,
          "model": "core.chaineditem",
          "fields": {
            "chain": [
              "1fs-article-default-forced-utf8"
            ],
            "parameters": "OrderedDict([('download',OrderedDict([\
('force_encoding', 'utf-8')]))])",
            "notes_fr": "...",
            "is_active": true,
            "check_error": null,
            "item_type": [
              "core",
              "processingchain"
            ],
            "is_valid": true,
            "notes_nt": "",
            "item_id": 2,
            "position": 0,
            "notes_en": ""
          }
        }
        """
        data = json.loads(data)
        obj = YAMLModel.objects.create(yaml=data)
        obj = YAMLModel.objects.get(id=obj.id)
        self.failUnlessEqual(obj.yaml, data)



"""
Custom YAML serializers that maintain their order
"""
import yaml
from collections import OrderedDict
from django.core.serializers.pyyaml import DjangoSafeDumper


class OrderedLoader(yaml.SafeLoader):
    pass


def construct_mapping(loader, node):
    loader.flatten_mapping(node)
    return OrderedDict(loader.construct_pairs(node))

OrderedLoader.add_constructor(
    yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
    construct_mapping
)


class OrderedDumper(DjangoSafeDumper):
    pass


def _dict_representer(dumper, data):
    return dumper.represent_mapping(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
        data.items()
    )

OrderedDumper.add_representer(OrderedDict, _dict_representer)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from distutils.core import Command


class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from django.conf import settings
        settings.configure(
            DATABASES={
                'default': {
                    'NAME': ':memory:',
                    'ENGINE': 'django.db.backends.sqlite3'
                }
            },
            MIDDLEWARE_CLASSES=(),
            INSTALLED_APPS=('yamlfield',)
        )
        from django.core.management import call_command
        import django
        django.setup()
        call_command('test', 'yamlfield')


setup(
    name='django-yamlfield',
    version='1.0.3',
    description='A Django database field for storing YAML data',
    author='The Los Angeles Times Data Desk',
    author_email='datadesk@latimes.com',
    url="http://django-yamlfield.readthedocs.io/",
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    install_requires=(
        'PyYAML>=3.10',
        'six>=1.4.1'
    ),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Framework :: Django',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'License :: OSI Approved :: MIT License',
    ],
    cmdclass={'test': TestCommand,}
)
